'''
Created on 2020-08-20

@author: wf
'''
import urllib.request
from bs4 import BeautifulSoup
import re

class WebScrape(object):
    '''
    WebScraper
    with a rudimentary Parser for https://en.wikipedia.org/wiki/RDFa
    extended for CEUR-WS and WikiCFP specific scraping
    
    https://stackoverflow.com/questions/21876602/what-does-the-html-typeof-attribute-do
    https://de.wikipedia.org/wiki/RDFa
    https://stackoverflow.com/questions/20767903/parsing-rdfa-in-html-xhtml
    https://www.w3.org/MarkUp/2009/rdfa-for-html-authors
    '''

    def __init__(self,debug:bool=False,showHtml:bool=False,timeout:float=20):
        '''
        Constructor
        
        Args:
            debug(bool): if True show debugging information
            showHtml(bool): if True show the HTML retrieved
            timeout(float): the default timeout 
        '''
        self.err=None
        self.valid=False
        self.debug=debug
        self.showHtml=showHtml
        self.timeout=timeout
        
    def findLinkForRegexp(self,regex:str):
        '''
        find a link for the given regular expression
        
        Args:
            regex(str): the regular expression to find a link for
        
        Return:
            m(object),text(str): the match/text tuple or None,None
        '''
        m=None
        text=None
        link=self.soup.find('a',href=re.compile(regex))
        if link:
            href=link['href']
            m=re.match(regex,href)    
            if hasattr(link, "text"):
                text=link.text 
        return m,text
        
    def fromTag(self,soup,tag,attr=None,value=None):
        '''
        get metadata from a given tag, attribute and value
        e.g. <span class="CEURVOLACRONYM">DL4KG2020</span>
        
        tag=span, attr=class, value=CEURVOLACRONYM
        
        Args:
           soup(BeautifulSoup): the parser to work with
           tag(string): the tag to search
           attr(string): the attribute to expect
           value(string): the value to expect
        '''
        # https://stackoverflow.com/a/16248908/1497139
        # find a list of all tag elements
        if attr is not None and value is not None:
            nodes = soup.find_all(tag, {attr : value})
        else:
            nodes = soup.find_all(tag)    
        lines = [node.get_text() for node in nodes]
        if len(lines)>0:
            return lines[0]
        else:
            return None
        
    def getSoup(self,url:str,showHtml:bool=False)->BeautifulSoup:
        '''
        get the beautiful Soup parser 
        
        Args:
           url(str): the url to open
           showHtml(boolean): True if the html code should be pretty printed and shown
           
        Return:
            BeautifulSoup: the html parser
        '''
        response = urllib.request.urlopen(url,timeout=self.timeout)
        html = response.read()
        soup = BeautifulSoup(html, 'html.parser', from_encoding='utf-8')  
        if showHtml:
            self.printPrettyHtml(soup)
            
        return soup    
    
    def printPrettyHtml(self,soup):
        '''
        print the prettified html for the given soup
        
        Args:
            soup(BeuatifulSoup): the parsed html to print
        '''
        prettyHtml=soup.prettify()
        print(prettyHtml)   
            
    def parseWithScrapeDescription(self,url,scrapeDescr=None):
        '''
        parse the given url with the given encoding
        
        Return:
             a dict with the results
        '''
        try:
            scrapeDict={}
            self.soup=self.getSoup(url, self.showHtml)        
            for scrapeItem in scrapeDescr:
                key=scrapeItem['key']
                tag=scrapeItem['tag']
                attr=scrapeItem['attribute']
                value=scrapeItem['value']
                value=self.fromTag(self.soup, tag,attr,value)
                scrapeDict[key]=value;  
            self.valid=True
        
        except urllib.error.HTTPError as herr:
            self.err=herr
        except urllib.error.URLError as terr:
            self.err=terr
        return scrapeDict
                
    def parseRDFa(self,url):
        '''
        rudimentary RDFa parsing
        '''
        triples=[]    
        try:
            self.soup=self.getSoup(url, self.showHtml)         
            subjectNodes = self.soup.find_all(True, {'typeof' : True})
            for subjectNode in subjectNodes:
                subject=subjectNode.attrs['typeof']
                if self.debug:
                    print(subjectNode)
                for predicateNode in subjectNode.find_all():
                    value=None 
                    name=None
                    if 'content' in predicateNode.attrs:
                        value=predicateNode.attrs['content']
                    else:
                        value=predicateNode.get_text()    
                    if 'property' in predicateNode.attrs:
                        name=predicateNode.attrs['property'] 
                    if name is not None and value is not None:
                        triples.append((subject,name,value))
            self.valid=True
        except urllib.error.HTTPError as herr:
            self.err=herr
        except urllib.error.URLError as terr:
            self.err=terr
        return triples    
    


"""
Created on 2020-08-20


  script to read event metadata from http://wikicfp.com
  
  use: python3 wikicfp.py [startId] [stopId] [threads]
  @param startId
  @param stopId
  @param threads - number of threads the script should create to improve performance
  
  example: python3 wikicfp.py --startId 2000 --stopId 2999 10

  @author:     svantje, wf
  @copyright:  2020-2021 TIB Hannover, Wolfgang Fahl. All rights reserved.

"""
import datasources.wikicfp 
from datasources.webscrape import WebScrape
from corpus.event import EventStorage
import datetime
from enum import Enum
import glob
import re
import os
import sys
import threading
import time
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
from lodstorage.jsonpicklemixin import JsonPickleMixin
import jsonpickle

class CrawlType(Enum):
    '''
    possible supported storage modes
    '''
    EVENT = "Event"      
    SERIES = "Series"
    
    @property
    def urlPrefix(self):
        baseUrl="http://www.wikicfp.com/cfp"
        if self==CrawlType.EVENT:
            url= f"{baseUrl}/servlet/event.showcfp?eventid="
        elif self==CrawlType.SERIES:
            url= f"{baseUrl}/program?id="
        return url
    
    @classmethod
    def valueMap(cls)->dict:
        '''
        get my list of values
        
        Return:
            list: the list of values
        '''
        valueMap={}
        for c in cls:
            valueMap[c.value]=c
        return valueMap
 
    @classmethod
    def isValid(cls,value:str)->bool:
        '''
        check whether the given value is valid
        
        Args:
            value(str): the value to check
        
        Return:
            bool: True if the value is a valid value of this enum
        '''
        valueMap=cls.valueMap()
        return value in valueMap

class CrawlBatch(object):
    '''
    describe a batch of pages to fetch metadata from
    '''
    
    def __init__(self,threads:int,startId:int,stopId:int,crawlTypeValue:str,threadIndex=None,):
        '''
        construct me with the given number of threads, startId and stopId
        
        Args:
           threads(int): number of threads to use
           startId(int): id of the event to start crawling with
           stopId(int): id of the event to stop crawling
           crawlTypeValue(str): the type of crawling (Event or Series)
           threadIndex(int): the index of this batch
        '''
        self.threads=threads
        self.threadIndex=threadIndex
        self.startId=startId
        self.stopId=stopId     
        if startId <= stopId: 
            self.step = +1
            self.total = stopId-startId+1
        else: 
            self.step = -1
            self.total = startId-stopId+1
        self.batchSize = self.total // threads
        if not CrawlType.isValid(crawlTypeValue):
            raise Exception(f"Invalid crawlType {crawlTypeValue}")
        self.crawlType=CrawlType.valueMap()[crawlTypeValue]
        
    def split(self)->list:
        '''
        split me for my threads
        '''
        crawlBatches=[]
        for threadIndex in range(self.threads):
            s = self.startId + threadIndex * self.batchSize
            e = s + self.batchSize-1
            splitBatch=CrawlBatch(1,s, e,self.crawlType.value,threadIndex)
            crawlBatches.append(splitBatch)
        return crawlBatches
      
    
    def __str__(self):
        '''
        get my string representation
        Return
            str: my string representation
        '''
        text=f"WikiCFP {self.crawlType.value} IDs {self.startId} - {self.stopId} ({self.threads} threads of {self.batchSize} IDs each)"
        return text
 
class WikiCfpScrape(object):
    '''
    support events from http://www.wikicfp.com/cfp/
    '''

    def __init__(self,config=None,debug:bool=False,jsondir:str=None,limit=200000,batchSize=1000,showProgress=True):
        '''
        Constructor
        
        Args:
            config(StorageConfig): the storage configuration to use
            debug(bool): if True debug for crawling is on
            limit(int): maximum number of entries to be crawled
            batchSize(int): default size of batches
            showProgress(bool): if True show Progress
        '''
        self.debug=debug
        self.limit=limit
        self.batchSize=batchSize
        self.showProgress=showProgress
        self.em=self.getEventManager(config)
        self.profile=self.em.config.profile
        if jsondir is not None:
            self.jsondir=jsondir
        else:
            cachePath=self.em.config.getCachePath()
            self.jsondir=f"{cachePath}/wikicfp"
            if not os.path.exists(self.jsondir):
                    os.makedirs(self.jsondir)
        
    def getEventManager(self,config=None,mode='sql'):
        '''
        get an EventManager
        
        Args:
            config(StorageConfig): the storage configuration to use
            mode(string): the storage mode to use e.g. "json" - will select a config based on mode if config is None
        '''
        if config is None:
            config=EventStorage.getStorageConfig(self.debug, mode)
        em=datasources.wikicfp.WikiCfpEventManager(config=config)
        return em
    
    def cacheEvents(self):
        '''
        cache my events to my eventmanager
        '''
        jsonEm=self.getEventManager(mode='json')
        if jsonEm.isCached():
            jsonEm.fromStore()
        else:    
            self.crawlFilesToJson(jsonEm)
        for event in jsonEm.events:
            self.em.events.append(event)
        self.em.store(limit=self.limit, batchSize=self.batchSize)    
        
    def crawlFilesToJson(self,jsonEm):    
        '''
        convert the crawlFiles to Json
        
        Args:
            jsonEm(EventManager): the JSON - storage based Eventmanager to use to collect the results
        '''
        # crawling is not done on startup but need to be done
        # in command line mode ... we just collect the json crawl result files here
        #self.crawl(startId=startId,stopId=stopId)
        startTime=time.time()
        jsonFiles=self.jsonFiles()
        if len(jsonFiles)==0:
            if self.profile or self.debug:
                print("No wikiCFP crawl json backups available")
                
        else:
            for jsonFilePath in jsonFiles:
                #batchEm=self.getEventManager(mode='jsonpickle')
                # legacy mode -file were created before ORM Mode
                # was available - TODO: make new files available in ORM mode with jsonable
                jsonPickleEm=JsonPickleMixin.readJsonPickle(jsonFileName=jsonFilePath,extension='.json')
                jsonPickleEvents=jsonPickleEm['events']
                if self.showProgress:
                    print("%4d: %s" % (len(jsonPickleEvents),jsonFilePath))
                for rawEvent in jsonPickleEvents.values():
                    event=datasources.wikicfp.WikiCfpEvent()
                    if 'title' in rawEvent and rawEvent['title'] is not None:
                        for field in rawEvent:
                            value=rawEvent[field]
                            # workaround:
                            # check jsonpickle __reduce__ values
                            if isinstance(value,dict):
                                # we need something like
                                # {"py/object": "datetime.datetime", "__reduce__": [{"py/type": "datetime.datetime"}, ["B+UHHwAAAAAAAA=="]]}
                                jsonValue=str(value).replace("'", '"')
                                dvalue=jsonpickle.decode(jsonValue)
                                value=dvalue
                            # make sure we ignore field like "py/object"
                            if not "py/" in field:
                                setattr(event,field,value)
                        event.source="wikicfp"
                        for field in ['startDate','endDate','locality','Submission_Deadline','Notification_Due','year']:
                            if not field in rawEvent:
                                setattr(event,field,None)
                        if event.startDate is not None:
                            event.year=event.startDate.year
                        event.url=WikiCfpEventFetcher.getUrl(event.wikiCFPId)
                        jsonEm.events.append(event)
            if self.profile:
                print ("read %d events in %5.1f s" % (len(jsonEm.events),time.time()-startTime))
            jsonEm.store(limit=self.limit,batchSize=self.batchSize)
        
    def jsonFiles(self):  
        '''
        get the list of the json files that have my data
        '''
        jsonFiles=sorted(glob.glob(f"{self.jsondir}/wikicfp_*.json"),key=lambda path:int(re.findall(r'\d+',path)[0]))
        return jsonFiles    
        
    def getJsonFileName(self,crawlBatch):
        '''
        get my the JsonFileName 
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl):
            
        Return:
            str: the json file name for this batch
        '''
        jsonFilePath=f"{self.jsondir}/wikicfp_{crawlBatch.crawlType.value}{crawlBatch.startId:06d}-{crawlBatch.stopId:06d}.json"
        return jsonFilePath
    
    def getBatchEntityManager(self,crawlBatch:CrawlBatch):
        '''
        get the batch Entity Manager for the given crawlBatch
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl):
            
        Return:
            EntityManager: either a Event or a Series Manager
        '''
        jsonFilepath=self.getJsonFileName(crawlBatch)
        config=EventStorage.getStorageConfig(debug=self.debug, mode="json")
        config.cacheFile=jsonFilepath
        crawlType=crawlBatch.crawlType
        if crawlType==CrawlType.EVENT:
            batchEm=datasources.wikicfp.WikiCfpEventManager(config=config)
        elif crawlType==CrawlType.SERIES:
            batchEm=datasources.wikicfp.WikiCfpEventSeriesManager(config=config)
        return batchEm
        
    def crawl(self,crawlBatch:CrawlBatch):
        '''
        see https://github.com/TIBHannover/confIDent-dataScraping/blob/master/wikicfp.py
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl
        '''
       
        print(f'crawling {crawlBatch}')
        batchEm=self.getBatchEntityManager(crawlBatch)
 
        # get all ids
        crawlType=crawlBatch.crawlType
        for eventId in range(int(crawlBatch.startId), int(crawlBatch.stopId+1), crawlBatch.step):
            wEvent=WikiCfpEventFetcher(crawlType=crawlType)
            rawEvent=wEvent.fromEventId(eventId)
            if crawlType == CrawlType.EVENT:
                event=datasources.wikicfp.WikiCfpEvent()
                event.fromDict(rawEvent)
                title="? deleted: %r" %event.deleted if not 'title' in rawEvent else event.title
                batchEm.getList().append(event)
            elif crawlType == CrawlType.SERIES:
                eventSeries=datasources.wikicfp.WikiCfpEventSeries()
                eventSeries.fromDict(rawEvent)
                title="?" if not 'title' in rawEvent else eventSeries.title
                batchEm.getList().append(eventSeries)
                
            print(f"{eventId:06d}: {title}")
           
           
        batchEm.store()
        return batchEm
            
    def threadedCrawl(self,crawlBatch:CrawlBatch):
        '''
        crawl with the given number of threads, startId and stopId
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl
        '''
        # determine the eventId range for each threaded job
        startTime=time.time()
        
        msg=f'Crawling {crawlBatch}'
        print(msg)

        # this list will contain all threads -> we can wait for all to finish at the end
        jobs = []

        # now start each thread with its id range and own filename
        for crawlBatch in crawlBatch.split(): 
        
            thread = threading.Thread(target = self.crawl, args=(crawlBatch,))
            jobs.append(thread)
            
        for job in jobs:
            job.start()   

        # wait till all threads have finished before print the last output
        for job in jobs:
            job.join()

        if self.debug:
            elapsed=time.time()-startTime
            print(f'crawling done after {elapsed:5.1f} s')
               
      
class WikiCfpEventFetcher(object):
    '''
    a single WikiCfpEentFetcher to fetch and event or series
    '''
    def __init__(self,crawlType=CrawlType.EVENT,debug=False,showProgress:bool=True,timeout=20):
        '''
        construct me
        
        Args:
            showProgress(bool): if True show progress
            timeout(float): the default timeout
        
        '''
        self.debug=debug
        self.crawlType=crawlType
        self.showProgress=showProgress
        self.progressCount=0
        self.timeout=timeout
            
    def fromTriples(self,rawEvent,triples): 
        '''
        get the rawEvent dict from the given triple e.g.:
        
        v:Event(v:summary)=IDC 2009
        v:Event(v:eventType)=Conference
        v:Event(v:startDate)=2009-06-03T00:00:00
        v:Event(v:endDate)=2009-06-05T23:59:59
        v:Event(v:locality)=Milano, Como, Italy
        v:Event(v:description)= IDC  2009 : The 8th International Conference on Interaction Design and Children
        v:Address(v:locality)=Milano, Como, Italy
        v:Event(v:summary)=Submission Deadline
        v:Event(v:startDate)=2009-01-19T00:00:00
        v:Event(v:summary)=Notification Due
        v:Event(v:startDate)=2009-02-20T00:00:00
        v:Event(v:summary)=Final Version Due
        v:Event(v:startDate)=2009-03-16T00:00:00
        '''
        recentSummary=None
        for s,p,o in triples:
            s=s.replace("v:","")
            p=p.replace("v:","")
            if self.debug:
                print ("%s(%s)=%s" % (s,p,o)) 
            if recentSummary in ['Submission Deadline','Notification Due','Final Version Due']:             
                key=recentSummary.replace(" ","_")
            else:
                key=p 
            if p.endswith('Date'):
                dateStr=o
                if dateStr=="TBD":
                    o=None
                else:    
                    o=datetime.datetime.strptime(
                        dateStr, "%Y-%m-%dT%H:%M:%S").date()
            if not key in rawEvent: 
                rawEvent[key]=o    
            if p=="summary": 
                recentSummary=o 
            else: 
                recentSummary=None
                
    @staticmethod       
    def getUrl(cfpid,crawlType:CrawlType=CrawlType.EVENT)->str:
        '''
        Args:
            cfpid(int): the WikiCFP id of the event or series
            crawlType(CrawlType): Event or Series
            
        Returns:
            the WikiCfP url
        '''
        url=f"{crawlType.urlPrefix}{cfpid}"
        return url   
    
    @classmethod
    def getLatestEvent(cls,debug=False,showProgress=True):
        '''
        get the latest Event doing a binary search
        '''
        wikicfp=WikiCfpEventFetcher(debug,showProgress=showProgress)
        wikicfp.progressCount=0
        wikicfp.getLatesEvetFromPair()
        
    def getHighestNonDeletedIdInRange(self,fromId:int,toId:int)->int:
        '''
        get the event with the highest id int the range fromId to toId that is not deleted
        
        Args:
            fromId(int): minimum id to search from
            toId(int): maximum id to search to
            
        Returns:
            int: maxium id of an event that is not deleted or None if there is none in this range
        '''
        maxId=None
        for eventId in range(fromId,toId+1):
            if self.showProgress:
                print(".",end='',flush=True)
                self.progressCount+=1
                if self.progressCount % 50 == 0:
                    print(flush=True)
            rawEvent=self.fromEventId(eventId)
            if not rawEvent['deleted']:
                maxId=eventId
        return maxId
    
    def getLatesEvetFromPair(self,low=5000,high=300000,margin=40):
        '''
        get the latest Event doing a binary search
        
        Args:
            low(int): lower index to search from
            hight(int): upper index boundary
        '''
        if high-low>margin+1:
            mid=(high+low)//2
            midId=(self.getHighestNonDeletedIdInRange(mid-margin, mid))
            if midId:
                return self.getLatesEvetFromPair(mid+1,high)
            else:
                return self.getLatesEvetFromPair(low, mid-1)
        else:
            return mid
        pass
                           
    def fromEventId(self,cfpid:int):
        '''
        see e.g. https://github.com/andreeaiana/graph_confrec/blob/master/src/data/WikiCFPCrawler.py
        
        Args:
            cfpid(int): the wikicfp id to use
        '''
        url=WikiCfpEventFetcher.getUrl(cfpid,self.crawlType)
        return self.fromUrl(url)
    
    def rawEventFromWebScrape(self,rawEvent:dict,triples:list,scrape:WebScrape):
        '''
        fill the given rawEvent with the data derived from the scrape 
        
        Args:
            rawEvent(dict): the event dictionary
            triples(list): the triples found
            scrape(WebScrape): the webscrape object to be used for parsing
        '''
        if len(triples)==0:
            #scrape.printPrettyHtml(scrape.soup)
            firstH3=scrape.fromTag(scrape.soup, 'h3')
            if "This item has been deleted" in firstH3:
                rawEvent['deleted']=True
        else:        
            self.fromTriples(rawEvent,triples)
            # add series information
            # Tag: <a href="/cfp/program?id=1769&amp;s=ISWC&amp;f=International Semantic Web Conference">International Semantic Web Conference</a>
            m,seriesText=scrape.findLinkForRegexp(r'/cfp/program\?id=([0-9]+).*')
            if m:
                seriesId=m.group(1)
                rawEvent['seriesId']=seriesId
                rawEvent['series']=seriesText
                pass
                
            if 'summary' in rawEvent:
                rawEvent['acronym']=rawEvent.pop('summary').strip()
            if 'description' in rawEvent:
                rawEvent['title']=rawEvent.pop('description').strip()
                
    def rawEventSeriesFromWebScrape(self,rawEvent:dict,scrape:WebScrape):
        '''
        fill the given rawEventSeries with the data derived from the scrape 
        
        Args:
            rawEvent(dict): the event dictionary
            scrape(WebScrape): the webscrape object to be used for parsing
        '''
        title=scrape.soup.find("title")
        if title:
            rawEvent["title"]=title.text.strip()
        dblpM,_text=scrape.findLinkForRegexp(r'http://dblp.uni-trier.de/db/(conf/[a-z0-9]+)/index.html')
        if dblpM:
            dblpSeriesId=dblpM.group(1)
            rawEvent['dblpSeriesId']=dblpSeriesId
        
         
        pass
 
    
    def fromUrl(self,url:str)->dict:
        '''
        get the event form the given url
        
        Args:
            url(str): the url to get the event from
        
        Returns:
            dict: a raw event dict or None if an error occured
        
        '''
        regexp=r"^"+self.crawlType.urlPrefix.replace("?","\?")+"(\d+)$"
        m=re.match(regexp,url)
        if not m:
            raise Exception("Invalid URL %s" % (url))
        else:
            cfpId=int(m.group(1))
        rawEvent={}
        if self.crawlType==CrawlType.EVENT:
            rawEvent['eventId']=f"{cfpId}" 
        else:
            rawEvent['seriesId']=f"{cfpId}" 
        rawEvent['wikiCfpId']=cfpId
        rawEvent['deleted']=False
        scrape=WebScrape(debug=self.debug,timeout=self.timeout)
        triples=scrape.parseRDFa(url)
        if scrape.err:
            raise Exception(f"fromUrl {url} failed {scrape.err}")
        if self.crawlType==CrawlType.EVENT:
            self.rawEventFromWebScrape(rawEvent, triples, scrape)
        else:
            self.rawEventSeriesFromWebScrape(rawEvent,scrape)
        return rawEvent
    
__version__ = 0.3
__date__ = '2020-06-22'
__updated__ = '2021-07-31'    

DEBUG = 1

    
def main(argv=None): # IGNORE:C0111
    '''main program.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)    
        
    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    user_name="Wolfgang Fahl"
    program_license = '''%s

  Created by %s on %s.
  Copyright 2020-2021 TIB Hannover, Wolfgang Fahl. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc,user_name, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument("-d", "--debug", dest="debug", action="count", help="set debug level [default: %(default)s]")
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        parser.add_argument('--startId', type=int, help='eventId to start crawling from', required=True)
        parser.add_argument('--stopId', type=int, help='eventId to stop crawling at', required=True)
        parser.add_argument('--crawlType',type=str,default="Event",help="The crawlType - Event or Series")
        parser.add_argument('-p','--targetPath',type=str,help="targetPath (JSON directory) for crawl results")
        parser.add_argument('-t','--threads', type=int, help='number of threads to start', default=10)

        # Process arguments
        args = parser.parse_args()
        wikiCfpScrape=WikiCfpScrape(jsondir=args.targetPath,debug=args.debug)
        crawlBatch=CrawlBatch(args.threads, args.startId, args.stopId,args.crawlType,None)
        wikiCfpScrape.threadedCrawl(crawlBatch)
        
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 1
    except Exception as e:
        if DEBUG:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2     
    
if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-d")
    sys.exit(main())



'''
Created on 2021-04-16

@author: wf
'''
from enum import Enum
from lodstorage.jsonable import JSONAble

class RatingType(str,Enum):
    '''
    the rating type
    
    see https://stackoverflow.com/a/51976841/1497139 why we use str as a type for better json Encoding
    '''
    missing='❌'
    invalid='👎'
    ok='👍'
    
class Rating(JSONAble):
    '''
    I am rating
    '''

    def __init__(self,pain:int=-1,reason:str=None,hint:str=None):
        '''
        Constructor
        
        Args:
            pain(int): the painLevel
            reason(str): the reason one of missing, invalid or ok
            hint(str): the description of the rating - hint on what is wrong
        '''
        self.set(pain,reason,hint)
        
    @classmethod
    def getSamples(cls):
        samplesLOD=[{
            "pain": 1,
            "reason": RatingType.ok,
            "hint": "Dates,  Jul 13, 2008 , Jul 14, 2008 valid",
        }
        ]
        return samplesLOD
        
    def set(self,pain:int,reason:str,hint:str):
        '''
        set my rating
        '''
        self.pain=pain
        self.reason=reason
        self.hint=hint
        
    def __str__(self):
        return f"{self.pain} - {self.reason}: {self.hint}"


'''
Created on 2020-07-11

@author: wf
'''
from corpus.event import Event,EventSeries,EventManager,EventSeriesManager
from lodstorage.storageconfig import StorageConfig
import html
import os
import json
from corpus.eventcorpus import EventDataSourceConfig,EventDataSource

class Confref(EventDataSource):
    sourceConfig=EventDataSourceConfig(lookupId="confref",name="confref.org",url="http://portal.confref.org",title="ConfRef",tableSuffix="confref")
    
    '''
    ConfRef platform
    '''
    def __init__(self):
        '''
        construct me 
        '''
        super().__init__(ConfrefEventManager(),ConfrefEventSeriesManager(),Confref.sourceConfig)

class ConfrefEvent(Event):
    '''
    a scientific event derived from Confref
    '''
    
    @staticmethod
    def fixRawEvent(rawEvent:dict):
        '''
        fix the given raw Event
        
        Args:
            rawEvent(dict): the raw event record to fix
        '''
        for key in rawEvent:
            value=rawEvent[key]
            if value is not None and type(value) is str:
                value=html.unescape(value)
                rawEvent[key]=value
        eventId=rawEvent.pop('id')
        area=rawEvent.pop('area')
        confSeries=rawEvent.pop('confSeries')
        rawEvent['eventId']=eventId
        rawEvent['url']=f'http://portal.confref.org/list/{eventId}'        
        rawEvent['title']=rawEvent.pop('name')
        rawEvent["source"]="confref"
        pass
    
    def fromDict(self,rawEvent:dict):
        '''
        get me from the given dict
        '''
        super().fromDict(rawEvent)
    
class ConfrefEventSeries(Event):
    '''
    a scientific event series derived from Confref
    '''
        
class ConfrefEventManager(EventManager):
    '''
    Crossref event manager
    '''
        
    def __init__(self, config: StorageConfig = None):
        '''
        Constructor
        '''
        super().__init__(name="ConfrefEvents", sourceConfig=Confref.sourceConfig, clazz=ConfrefEvent, config=config)
        
    def configure(self):
        '''
        configure me
        '''
        # nothing to do - there is a get ListOfDicts below
    
    def getListOfDicts(self):
        '''
        get my content from the json file
        '''
        cachePath=self.config.getCachePath()
        jsondir=f"{cachePath}/confref"
        if not os.path.exists(jsondir):
                os.makedirs(jsondir)
        self.jsonFilePath=f"{jsondir}/confref-conferences.json"
        with open(self.jsonFilePath) as jsonFile:
            rawEvents=json.load(jsonFile)
        lod=[]
        for rawEvent in rawEvents:
            ConfrefEvent.fixRawEvent(rawEvent)
            lod.append(rawEvent)
        return lod
        
class ConfrefEventSeriesManager(EventSeriesManager):
    '''
    Confref event series handling
    '''
    
    def __init__(self,config:StorageConfig=None):
        '''
        Constructor
        '''
        super().__init__(name="ConfrefEventSeries", sourceConfig=Confref.sourceConfig,clazz=ConfrefEventSeries,config=config)


    def configure(self):
        '''
        configure me
        '''
        # nothing to do getListOfDicts is defined
        
    def getListOfDicts(self):
        '''
        get my data
        '''
        # TODO Replace this stub
        lod=[{'source':'crossref','eventSeriesId':'dummy'}]
        return lod


'''
Created on 2021-04-16

@author: wf
'''

class PainScale(object):
    '''
    painscale handling
    '''

    @staticmethod
    def lookupPainImage(rating: int,size=64,asImageTag=True):
        '''
        Returns html image tag to the corresponding pain rating
       
        Args:
            rating(int): the pain rating
        '''
        painImages = {
                      0: f"https://upload.wikimedia.org/wikipedia/commons/thumb/4/46/Pain_0_png_rendered.png/{size}px-Pain_0_png_rendered.png", 
                      1: f"https://upload.wikimedia.org/wikipedia/commons/thumb/1/12/Pain_1_png_rendered.png/{size}px-Pain_1_png_rendered.png",
                      2: f"https://upload.wikimedia.org/wikipedia/commons/thumb/0/04/Pain_2_png_rendered.png/{size}px-Pain_2_png_rendered.png",
                      3: f"https://upload.wikimedia.org/wikipedia/commons/thumb/0/04/Pain_2_png_rendered.png/{size}px-Pain_2_png_rendered.png",
                      4: f"https://upload.wikimedia.org/wikipedia/commons/thumb/6/6f/Pain_3_png_rendered.png/{size}px-Pain_3_png_rendered.png",
                      5: f"https://upload.wikimedia.org/wikipedia/commons/thumb/b/bb/Pain_5_png_rendered.png/{size}px-Pain_5_png_rendered.png",
                      6: f"https://upload.wikimedia.org/wikipedia/commons/thumb/e/e7/Pain_6_png_rendered.png/{size}px-Pain_6_png_rendered.png",
                      7: f"https://upload.wikimedia.org/wikipedia/commons/thumb/a/a8/Pain_7_png_rendered.png/{size}px-Pain_7_png_rendered.png",
                      8: f"https://upload.wikimedia.org/wikipedia/commons/thumb/d/d0/Pain_8_png_rendered.png/{size}px-Pain_8_png_rendered.png",
                      9: f"https://upload.wikimedia.org/wikipedia/commons/thumb/2/24/Pain_9_png_rendered.png/{size}px-Pain_9_png_rendered.png",
                     10: f"https://upload.wikimedia.org/wikipedia/commons/thumb/7/78/Pain_10_png_rendered.png/{size}px-Pain_10_png_rendered.png"
                      }
        if rating >= 0 and rating <= 10:
            src=painImages[rating]
            if asImageTag:
                return f'<img alt="{rating}" src="{src}" width="{size}" height="{size}"/>'
            else:
                return src
        else:
            return ""


'''
Created on 2021-07-31

@author: wf
'''

from corpus.event import EventSeriesManager, EventSeries, Event, EventManager
from lodstorage.storageconfig import StorageConfig
import datasources.wikicfpscrape
from corpus.eventcorpus import EventDataSource, EventDataSourceConfig


class WikiCfp(EventDataSource):
    '''
    scientific event from http://www.wikicfp.com
    '''
    sourceConfig = EventDataSourceConfig(lookupId="wikicfp", name="WikiCFP", url='http://www.wikicfp.com', title='WikiCFP', tableSuffix="wikicfp")
    
    def __init__(self):
        '''
        constructor
        '''
        super().__init__(WikiCfpEventManager(), WikiCfpEventSeriesManager(), WikiCfp.sourceConfig)

        
class WikiCfpEventSeries(EventSeries):
    '''
    event series derived from WikiCFP
    '''
    
    
class WikiCfpEvent(Event):
    '''
    event derived from WikiCFP
    '''
    
    @classmethod
    def getSamples(cls):
        samples = [
        {
            "Notification_Due": "2008-05-29",
            "Submission_Deadline": "2008-04-15",
            "acronym": "SIGMAP 2008",
            "city": "Porto",
            "country": "Portugal",
            "deleted": False,
            "endDate": "2008-07-29",
            "eventType": "Conference",
            "homepage": "http://www.sigmap.org/SIGMAP2008/CallforPapers.html",
            "locality": "Poato, Portugal",
            "lookupAcronym": "SIGMAP 2008",
            "source": "wikicfp",
            "startDate": "2008-07-26",
            "title": "SIGMAP  2008 : International Conference on Signal Processing and Multimedia Applications",
            "url": "http://www.wikicfp.com/cfp/servlet/event.showcfp?eventid=977",
            "wikiCFPId": 977,
            "year": 2008
        }]
        return samples
    
    
class WikiCfpEventManager(EventManager):
    '''
    manage WikiCFP derived scientific events
    '''

    def __init__(self, config:StorageConfig=None):
        '''
        Constructor
        '''
        super().__init__(name="WikiCfpEvents", sourceConfig=WikiCfp.sourceConfig,clazz=WikiCfpEvent, config=config)
        
    def configure(self):
        '''
        configure me
        '''
        # no need - getListOfDicts has a specialized implementation
        pass
        
    def getListOfDicts(self):
        '''
        get my list of dicts
        '''
        wikiCFP = datasources.wikicfpscrape.WikiCfpScrape()
        if not wikiCFP.em.isCached():
            wikiCFP.cacheEvents()
        else:
            wikiCFP.em.fromStore()
        lod = []
        for event in wikiCFP.em.events:
            lod.append(event.__dict__)
        return lod    

 
class WikiCfpEventSeriesManager(EventSeriesManager):
    '''
    mange WikiCFP derived scientific conference series
    '''

    def __init__(self, config:StorageConfig=None):
        '''
        Constructor
        '''
        super(WikiCfpEventSeriesManager, self).__init__(name="WikiCfpEventSeries", sourceConfig=WikiCfp.sourceConfig, clazz=WikiCfpEventSeries, config=config)
        
    def configure(self):
        '''
        configure me
        '''
        # TODO implement
        
    def getListOfDicts(self):
        '''
        get my list of dicts
        '''
        lod = [{
            "acronym": "ESWC",
            "wikiCfpId": 933,
            "title": "Extended Semantic Web Conference"
        }]
        # TODO implement
        # return an empty list for the time being
        return lod



'''
Created on 2021-07-21

@author: wf
'''
from corpus.event import EventSeriesManager,EventSeries, Event, EventManager
from lodstorage.sparql import SPARQL
from lodstorage.storageconfig import StorageConfig
from corpus.eventcorpus import EventDataSource,EventDataSourceConfig

class Wikidata(EventDataSource):
    '''
    Wikidata access via SPARQL endpoint
    
    make do not want to be dependend on this endpoint since we might have
    our own copy of Wikidata which might run on Virtuoso or Jena instead of blazegraph
    '''
    endpoint="https://query.wikidata.org/sparql"
    sourceConfig=EventDataSourceConfig(lookupId="wikidata",name="Wikidata",url='https://www.wikidata.org/wiki/Wikidata:Main_Page',title='Wikidata',tableSuffix="wikidata")
    
    def __init__(self):
        '''
        construct me
        '''
        super().__init__(WikidataEventManager(),WikidataEventSeriesManager(),Wikidata.sourceConfig)
        
class WikidataEventSeries(EventSeries):
    '''
    event series derived from Wikidata
    '''
    
class WikidataEvent(Event):
    '''
    event derived from Wikidata
    '''
    
    @staticmethod
    def fixRawEvent(rawEvent:dict):
        '''
        fix the given raw Event
        
        Args:
            rawEvent(dict): the raw event record to fix
        '''
        if 'startDate' in rawEvent:
            startDate=rawEvent['startDate']
            if startDate:
                rawEvent['year']=startDate.year
                pass
    
class WikidataEventManager(EventManager):
    '''
    manage wikidata derived scientific events
    '''
    def __init__(self,config:StorageConfig=None):
        '''
        Constructor
        '''
        super(WikidataEventManager,self).__init__(name="WikidataEvents", sourceConfig=Wikidata.sourceConfig,clazz=WikidataEvent,config=config)
        
    def configure(self):
        '''
        configure me
        '''
        if not hasattr(self, "getListOfDicts"):
            self.getListOfDicts=self.getLoDfromEndpoint    
   
    def getSparqlQuery(self):
        '''
        get  the SPARQL query for this series
        
        see also 
           https://github.com/TIBHannover/confiDent-dataimports/blob/master/wip/wikidata_academic_conferences.rq
           https://confident.dbis.rwth-aachen.de/or/index.php?title=Iteration1_Property_Mapping
        '''
        query="""PREFIX wd: <http://www.wikidata.org/entity/>
PREFIX wdt: <http://www.wikidata.org/prop/direct/>
PREFIX wikibase: <http://wikiba.se/ontology#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>

SELECT DISTINCT 
  (?event as ?eventId)
  (?event as ?url)
  ?acronym
  ?ordinal
  #?eventLabel
  ?title
  ?locationLabel 
  #?countryIdLabel
  ?country
  ?countryId
  ?eventInSeries
  ?eventInSeriesId
  ?startDate
  ?endDate
  ?homepage 
  ?dblpConferenceId
  ?wikiCfpId
  ?gndId
  ?mainSubject
  ?language
 
WHERE
{  
 
  # wdt:P31 (instance of)  wd:Q52260246 (scientific event)
  # Q2020153 (academic conference)
  ?event wdt:P31 wd:Q2020153 .
  # acronym
  OPTIONAL { ?event wdt:P1813 ?acronym }
  # ordinal
  OPTIONAL { ?event wdt:P1545 ?ordinal }
  # properties with type:literal # requiring label
  OPTIONAL { ?event wdt:P276 ?location . }
  OPTIONAL { 
      ?event wdt:P17 ?countryId . 
      ?countryId rdfs:label ?country filter (lang(?country)   = "en").
  }
  OPTIONAL {
      ?event wdt:P276 ?locationId.
      ?locationId rdfs:label ?location filter (lang(?location)   = "en").
  }
  OPTIONAL { 
    ?event wdt:P179 ?eventInSeriesId . 
    ?eventInSeriesId rdfs:label ?eventInSeries filter (lang(?eventInSeries)   = "en").
  }
  OPTIONAL { 
    ?event wdt:P2936 ?languageId .
    ?languageId rdfs:label ?language filter (lang(?language)   = "en").
  }
  OPTIONAL { 
    ?event wdt:P921 ?mainSubjectId .
    ?mainSubjectId rdfs:label ?mainSubject filter (lang(?mainSubject)   = "en").
  }
  OPTIONAL { ?event wdt:P580 ?startDate . }
  OPTIONAL { ?event wdt:P582 ?endDate . }
  
  OPTIONAL { ?event wdt:P856 ?homepage . }
  OPTIONAL { ?event wdt:P8926 ?dblpConferenceId . } 
  OPTIONAL { ?event wdt:P5124 ?wikiCfpId . }
  OPTIONAL { ?event wdt:P227 ?gndId. }
  OPTIONAL { ?event wdt:P214 ?viafId. }
  # labels 
  # works only with WikiData Query Service / blazegraph
  # SERVICE wikibase:label { bd:serviceParam wikibase:language "en". } # provide Label in EN        
  ?event rdfs:label ?title filter (lang(?title)   = "en").
  
}
"""
        return query
    
    def getLoDfromEndpoint(self,endpoint=Wikidata.endpoint):
        '''
        get my content from the given endpoint
        '''
        sparql=SPARQL(endpoint)
        query=self.getSparqlQuery()
        listOfDicts=sparql.queryAsListOfDicts(query)
        self.setAllAttr(listOfDicts,"source","dblp")
        for rawEvent in listOfDicts:
            WikidataEvent.fixRawEvent(rawEvent)
        return listOfDicts
    
class WikidataEventSeriesManager(EventSeriesManager):
    '''
    wikidata scientific conference Series Manager
    '''

    def __init__(self,config:StorageConfig=None):
        '''
        Constructor
        '''
        super(WikidataEventSeriesManager,self).__init__(name="WikidataEventSeries", sourceConfig=Wikidata.sourceConfig,clazz=WikidataEventSeries,config=config)
        
    def configure(self):
        '''
        configure me
        '''
        if not hasattr(self, "getListOfDicts"):
            self.getListOfDicts=self.getLoDfromEndpoint
        
    def getSparqlQuery(self):
        '''
        get  the SPARQL query for this series
        '''
        query="""
        # Conference Series wikidata query
        # see https://confident.dbis.rwth-aachen.de/dblpconf/wikidata
        # WF 2021-01-30
        PREFIX wd: <http://www.wikidata.org/entity/>
        PREFIX wdt: <http://www.wikidata.org/prop/direct/>
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        SELECT (?confSeries as ?eventSeriesId) ?acronym ?confSeriesLabel ?official_website ?DBLP_pid ?WikiCFP_pid ?FreeBase_pid ?Microsoft_Academic_pid ?Publons_pid ?ACM_pid ?GND_pid
        WHERE 
        {
          #  scientific conference series (Q47258130) 
          ?confSeries wdt:P31 wd:Q47258130.
          OPTIONAL { ?confSeries wdt:P1813 ?short_name . }
          BIND (COALESCE(?short_name,?confSeriesLabel) AS ?acronym).
          #  official website (P856) 
          OPTIONAL {
            ?confSeries wdt:P856 ?homepage
          } 
          # any item with a DBLP venue ID 
          OPTIONAL {
            ?confSeries wdt:P8926 ?DBLP_pid.
          }
          # WikiCFP pid 
          optional {
             ?confSeries wdt:P5127 ?WikiCFP_pid.
          }
          # FreeBase pid
          optional {
              ?confSeries wdt:P646 ?FreeBase_pid.
          }
          # Microsoft Academic ID
          optional {
              ?confSeries wdt:P6366 ?Microsoft_Academic_pid.
          }
          # Publons journals/conferences ID 
          optional {
              ?confSeries wdt:P7461 ?Publons_pid.
          }
          # ACM conference ID   
          optional {
            ?confSeries wdt:P7979 ?ACM_pid.
          }
          # GND pid
          optional {
            ?confSeries wdt:P227 ?GND_pid.
          }
          # label 
          ?confSeries rdfs:label ?confSeriesLabel filter (lang(?confSeriesLabel) = "en").
        }
        ORDER BY (?acronym)
"""
        return query

    def getLoDfromEndpoint(self,endpoint=Wikidata.endpoint)->list:
        '''
        get my content from the given endpoint
        '''
        sparql=SPARQL(endpoint)
        query=self.getSparqlQuery()
        listOfDicts=sparql.queryAsListOfDicts(query)
        self.setAllAttr(listOfDicts,"source","wikidata")
        return listOfDicts
            
        
        